import { EditorState, Transaction, ChangeSet, Facet, Extension } from "../../state";
import { StyleModule } from "style-mod";
import { DecorationSet } from "./decoration";
import { EditorView } from "./editorview";
import { Attrs } from "./attributes";
import { Rect } from "./dom";
export declare type Command = (target: EditorView) => boolean;
export declare const domEventHandlers: Facet<{
    [key: string]: (view: EditorView, event: any) => boolean;
}, readonly {
    [key: string]: (view: EditorView, event: any) => boolean;
}[]>;
export declare const clickAddsSelectionRange: Facet<(event: MouseEvent) => boolean, readonly ((event: MouseEvent) => boolean)[]>;
export declare const dragMovesSelection: Facet<(event: MouseEvent) => boolean, readonly ((event: MouseEvent) => boolean)[]>;
export declare class ViewPlugin {
    update(update: ViewUpdate): void;
    destroy(): void;
    decorations: DecorationSet;
    scrollMargins: Partial<Rect> | null;
    static get extension(this: {
        new (view: EditorView): ViewPlugin;
    }): Extension;
    static dummy: {
        update(update: ViewUpdate): void;
        destroy(): void;
        decorations: DecorationSet;
        scrollMargins: Partial<Rect> | null;
    };
}
export interface MeasureRequest<T> {
    key?: any;
    read(view: EditorView): T;
    write(measure: T, view: EditorView): void;
}
export declare const viewPlugin: Facet<(view: EditorView) => ViewPlugin, readonly ((view: EditorView) => ViewPlugin)[]>;
export declare const editorAttributes: Facet<Attrs, Attrs>;
export declare const contentAttributes: Facet<Attrs, Attrs>;
export declare const decorations: Facet<DecorationSet, readonly DecorationSet[]>;
export declare const styleModule: Facet<StyleModule<{}>, readonly StyleModule<{}>[]>;
export declare const phrases: Facet<{
    [key: string]: string;
}, readonly {
    [key: string]: string;
}[]>;
export declare const enum UpdateFlag {
    Focus = 1,
    Height = 2,
    Viewport = 4,
    Oracle = 8
}
export declare class ViewUpdate {
    readonly view: EditorView;
    readonly state: EditorState;
    readonly transactions: readonly Transaction[];
    readonly changes: ChangeSet;
    readonly prevState: EditorState;
    flags: number;
    constructor(view: EditorView, state: EditorState, transactions?: readonly Transaction[]);
    get viewportChanged(): boolean;
    get heightChanged(): boolean;
    get focusChanged(): boolean;
    get docChanged(): boolean;
    get selectionSet(): boolean;
    get empty(): boolean;
}
//# sourceMappingURL=extension.d.ts.map